unit MENetwork;

interface

uses MMatrix,MComplex,Classes;

type
     PBranch=^TBranch;
     Pnode=^TNode;

     TBranch=record
      tp:integer;              //  
      b:string[5];             //  
      e:string[5];             //  
      z:tcomplex;              //  
      gb:tcomplex;             // 
      eds:tcomplex;            //  ( )
      kt:extended;             // .  ( )
      from:integer;            //   (    )
      into:integer;            //    ---\---\---\--
      len:Double;            //  
      fixed:boolean;
      Selected:boolean;
     end;


     TNode=record
      x:Double;              //    X
      y:Double;              // ---/---/----/---  Y
      dx:Double;             //  X
      dy:Double;             //  Y
      fixed:boolean;           //   ( )
      caption:string[5];       //  
      SelfY:Tcomplex;          //  
      SelfI:Tcomplex;          //  
      SelfS:Tcomplex;          //  
      SelfU:Tcomplex;          //  voltage
      SelfF:TComplex;
      Selected:boolean;
     end;

     TENetwork=class(TObject)
     private
      BranchItem:PBranch;
      NodeItem:PNode;

      FBranchCount:integer;
      FnodeCount:integer;

      MatrixY:TMatrix;
      MatrixZ:TMatrix;
      MatrixI:TMatrix;
      MatrixUB:TMatrix;
      MatrixIkz:Tmatrix;
      MatrixUa:Tmatrix;

      FKUdR:Extended; //     R=0
      FKUdX:Extended; //     X=0

      function GetBranches(V:integer):TBranch;
      procedure SetBranches(V:Integer;Value:TBranch);
      function GetNodes(V:integer):TNode;
      procedure SetNodes(V:Integer;Value:TNode);

     public
      BranchItems:Tlist;
      NodeItems:Tlist;

      constructor Create;
      Destructor Destroy;override;

      property KudR:Extended read FKUdr write FKudr;
      property KudX:Extended read FKUdx write FKudx;

      property Z:TMatrix read MatrixZ;
      property Y:TMatrix read MatrixY;
      property I:TMatrix read MatrixI;
      property Ub:TMatrix read MatrixUb;
      property Ikz:TMatrix read MatrixIkz;
      property Ua:TMatrix read MatrixUa;

      property BranchCount:integer read FBranchCount;
      property NodeCount:integer read FNodeCount;

      property Branches[V:integer]:TBranch read GetBranches write SetBranches;
      property Nodes[V:integer]:TNode read GetNodes write SetNodes;

      procedure AddBranch(t1:integer;nb,ne:string;z1,z2,ed:tcomplex;kk:extended;Ln1:extended);
      procedure AddNode(nb:string;y1,i1,s,u:tcomplex;v:extended);
      procedure GenerateNodeFromBranch;
      procedure LinkBranchWithNode;
      procedure GenerateYMatrix;
      procedure CalculateVoltageBeforeKZ;
      procedure CalculateNodeI;
      procedure CalculateVoltageAfterKZ;
      procedure CalculateAll;
      function FindNodeByName(ss:string):integer;
      Procedure SeidelVoltage(ki:integer);
      function CalcTa(j:integer):Extended;
      function CalcKud(j:integer):Extended;
      function CalcIud(j:integer):Extended;
     end;

const s3=1.73205080756887729352744634150587;
      s2=1.4142135623730950488016887242097;

implementation

uses Math,SYSUtils;

function TENetwork.CalcTa(j:integer):Extended;
begin
if z[j,j].r<>0 then Result:=TakeImagePart(z[j,j])/(TakeRealPart(z[j,j])*314)
 else Result:=0;
end;

function TENetwork.CalcKud(j:integer):Extended;
begin
if z[j,j].r=0 then Result:=KudR else
 if z[j,j].i=0 then Result:=KudX else
 Result:=1+exp(-0.01/CalcTa(j));
end;

function TENetwork.CalcIud(j:integer):Extended;
begin
 Result:=ModuleComplex(Ikz[1,j])*s2*CalcKud(j);
end;

constructor TENetwork.Create;
begin
MatrixY:=Tmatrix.Create(3,3);
MatrixZ:=Tmatrix.Create(3,3);
MatrixI:=Tmatrix.Create(3,3);
MatrixUB:=Tmatrix.Create(3,3);
MatrixIkz:=Tmatrix.Create(3,3);
MatrixUa:=Tmatrix.Create(3,3);

FKUdR:=1.7;
FKUdX:=1;

BranchItems:=TList.create;
NodeItems:=TList.create;
BranchItems.clear;
NodeItems.clear;
FBranchCount:=0;
FNodeCount:=0;
end;

Destructor TENetwork.Destroy;
var i:integer;
begin
with BranchItems do
for i:=0 to count-1 do
 begin
 BranchItem:=items[i];
 dispose(BranchItem);
 end;
with NodeItems do
for i:=0 to count-1 do
 begin
 NodeItem:=items[i];
 dispose(NodeItem);
 end;
BranchItems.free;
NodeItems.free;
MatrixY.free;
MatrixZ.Free;
MatrixI.free;
MatrixUB.free;
MatrixIkz.free;
MatrixUa.free;
inherited destroy;
end;

function TENetwork.GetBranches(V:integer):TBranch;
begin
BranchItem:=BranchItems.items[v];
result:=BranchItem^;
end;

procedure TENetwork.SetBranches(V:Integer;Value:TBranch);
begin
//new(BranchItem);
BranchItem:=BranchItems.items[V];
BranchItem^:=value;

end;

function TENetwork.GetNodes(V:integer):TNode;
begin
NodeItem:=NodeItems.items[v];
result:=NodeItem^;
end;

procedure TENetwork.SetNodes(V:Integer;Value:TNode);
begin
//new(NodeItem);
NodeItem:=NodeItems.items[V];
NodeItem^:=value;
NodeItems.items[V]:=NodeItem;
end;

procedure TENetwork.AddBranch(t1:integer;nb,ne:string;z1,z2,ed:tcomplex;kk:extended;Ln1:extended);
begin
new(BranchItem);
BranchItem^.tp:=t1;
BranchItem^.b:=nb;
BranchItem^.e:=ne;
BranchItem^.z:=z1;
BranchItem^.gb:=z2;
BranchItem^.eds:=ed;
if kk<>0 then BranchItem^.kt:=kk else BranchItem^.kt:=1;
BranchItem^.Len:=ln1;
BranchItems.add(BranchItem);
inc(FBranchCount);
end;

procedure TENetwork.AddNode(nb:string;y1,i1,s,u:tcomplex;v:extended);
begin
new(NodeItem);
NodeItem^.caption:=nb;
NodeItem^.SelfY:=y1;
NodeItem^.SelfI:=i1;
NodeItem^.SelfS:=s;
NodeItem^.SelfU:=u;
NodeItem^.SelfF:=Makecomplex(v,0);
NodeItems.add(NodeItem);
inc(FNodeCount);
end;

function TENetwork.FindNodeByName(ss:string):integer;
var i:integer;
begin
 for i:=1 to NodeItems.count do
  begin
  NodeItem:=NodeItems.items[i-1];
  if NodeItem.Caption=ss then
    begin
    result:=i-1;
    exit;
    end;
  end;
result:=-1;
end;

procedure TENetwork.GenerateNodeFromBranch;
var i:integer;
begin
 for i:=0 to BranchItems.count-1 do
  begin
  BranchItem:=BranchItems.Items[i];
  if (FindNodeByName(BranchItem^.b)=-1) and (BranchItem^.b<>'0') then AddNode(BranchItem^.b,NullComplex,NullComplex,NullComplex,MakeCOmplex(220,0),220);
  if (FindNodeByName(BranchItem^.e)=-1) and (BranchItem^.e<>'0') then AddNode(BranchItem^.e,NullComplex,NullComplex,NullComplex,MakeCOmplex(220,0),220);
  end;
end;

procedure TENetwork.LinkBranchWithNode;
var i:integer;
begin
 for i:=0 to BranchItems.count-1 do
  begin
  BranchItem:=BranchItems.Items[i];
  BranchItem^.from:=FindNodeByName(BranchItem^.b);
  BranchItem^.into:=FindNodeByName(BranchItem^.e);
  end;
end;

procedure TENetwork.GenerateYMatrix;
var i,x,y:integer;
    zy,kt,gb:Tcomplex;
begin
MatrixY.recreateMatrix(Nodecount,nodecount);
MatrixI.recreateMatrix(1,Nodecount);
matrixY.fillMatrix(0);
for i:=0 to BranchCount-1 do
  begin
  x:=Branches[i].from+1;
  y:=Branches[i].Into+1;
  zy:=InversComplex(Branches[i].z);
  kt:=MakeComplex(Branches[i].kt,0);
  gb:=Branches[i].gb;
if (x<>0) and (y<>0) then
      begin
      MatrixY[x,y]:=AddComplex(MatrixY[x,y] , DivComplex( zy , kt ));
      MatrixY[y,x]:=AddComplex(MatrixY[y,x] , DivComplex( zy , kt ));
      end;
  if y<>0 then MatrixY[y,y]:=SubComplex( MatrixY[y,y] , Addcomplex(DivComplex( zy , MulComplex(kt,kt) ),gb) );
  if x<>0 then MatrixY[x,x]:=SubComplex( MatrixY[x,x] , AddComplex(zy,gb) )
   else
    begin
    MatrixI.Items[1,y]:=SubComplex(MatrixI.Items[1,y], MulComplex(Branches[i].eds,zy))
{    NodeItem:=NodeItems.items[y-1];
    nodeitem^.SelfI:=SubComplex(nodeitem^.SelfI, MulComplex(Branches[i].eds,zy))}
    end;
  end;
end;

procedure TENetwork.CalculateVoltageBeforeKZ;
begin
MatrixZ.CopyFromMatrix(MatrixY);
MatrixZ.InvertMatrix;
MulMatrixEX(MatrixZ,MatrixI,MatrixUB);
end;

procedure TENetwork.CalculateNodeI;
var i:integer;
begin
MatrixIkz.recreateMatrix(1,Nodecount);
for i:=1 to MatrixIkz.RowCOunt do MatrixIkz[1,i]:=DivComplex(Ub[1,i],NegativeComplex(MatrixZ[i,i]));
//MatrixIkz.DivMatrixPerComplex(MakeComplex(s3,0));
end;

procedure TENetwork.CalculateVoltageAfterKZ;
var Temp1,Temp2:Tmatrix;
    I,j:integer;
begin
MatrixUA.RecreateMatrix(nodecount,nodecount);
Temp1:=TMatrix.Create(1,NodeCount);
Temp2:=TMatrix.Create(1,NodeCount);
for i:=1 to NodeCount do
  begin
  Temp1.CopyFromMatrix(MatrixI);
  Temp1[1,i]:=AddComplex(Temp1[1,i], {DivComplex(MatrixUB[1,i],Matrix[i,i])}MatrixIkz[1,i] );
  MulMatrixEX(MatrixZ,Temp1,Temp2);
//  Temp2.DivMatrixPerComplex(MakeComplex(sqrt(3),0));
  for j:=1 to nodeCount do
      if ModuleComplex(Temp2[1,j])>1E-15 then  MatrixUA[i,j]:=Temp2[1,j];
      //MakeCOmplex(ModuleComplex(Temp2[1,j]),0);
  Temp2.fillmatrix(0);
  end;
Temp2.free;
Temp1.free;
end;

procedure TENetwork.CalculateAll;
begin
GenerateNodeFromBranch;
linkBranchWithNode;
generateYMatrix;
CalculateVoltageBeforeKZ;
CalculateNodeI;
CalculateVoltageAfterKZ;
end;

Procedure TENetwork.SeidelVoltage(ki:integer);
var i,j,ki1,l:integer;
    A,B,V:Extended;
    t1,t2:Tcomplex;
begin
for l:=1 to ki do
 for i:=1 to NodeCount-1 do
begin
A:=0;
B:=0;
 for j:=1 to NodeCount do
  if j<>i then
    if not COmplexEqualZero(MatrixY.items[j,i]) then
    begin
    A:=A+MatrixY[j,i].r*Nodes[j-1].SelfU.r+MatrixY[j,i].i*Nodes[j-1].SelfU.i;
    B:=B+MatrixY[j,i].r*Nodes[j-1].SelfU.i+MatrixY[j,i].i*Nodes[j-1].SelfU.r;
    end;
// with Nodes[i].selfI:=Nodes[i].SelfS.r+Nodes[i].SelfU.r+Nodes[i].SelfS.i+Nodes[i].SelfU.i
  with Nodes[i-1] do
    begin
    t1:=MakeComplex(
       (SelfS.r*SelfU.r+SelfS.i*SelfU.i)/Power(ModuleComplex(SelfU),2),
       (SelfS.r*SelfU.i-SelfS.i*SelfU.r)/Power(ModuleComplex(SelfU),2)
       );
    t2:=MakeCOmplex(
       (MatrixY[i,i].r*(t1.r-A)+MatrixY[i,i].i*(t1.i-B))/(Power(MatrixY[i,i].r,2)+Power(MatrixY[i,i].i,2)),
       (MatrixY[i,i].r*(t1.i-B)-MatrixY[i,i].i*(t1.r-A))/(Power(MatrixY[i,i].r,2)+Power(MatrixY[i,i].i,2))
       );

    NodeItem:=NodeItems.items[i-1];
    NodeItem^.SelfI:=t1;
    NodeItem^.SelfU:=t2;
    A:=0;
    B:=0;
   // Form1.Stringgrid2.Cells[l+1,i]:=FloatTOstr(ModuleComplex(t2));
    V:=ModuleComplex(Nodeitem^.SelfU);
    NodeItem^.SelfU:=MakeComplex((NodeItem^.SelfU.r*NodeItem^.SelfF.r)/V,(NodeItem^.SelfU.i*NodeItem^.SelfF.r)/V);
    end;
end;


end;


end.
